class Evaluation:

    def generalized_random_forest_confidence_intervals(self): # import as grf_ci
        GRF_CIs = 1
        return GRF_CIs

    def qini_AUUC_scores(self, qini = True):
        qini_AUUC = 1
        return qini_AUUC